import java.util.*;

import com.ibm.mq.MQException;
import com.ibm.mq.MQMessage;
import com.ibm.mq.constants.CMQC;

/**
 * MD class to hold all of a message's message descriptor values.
 * 
 * @author Roger Lacroix, Capitalware Inc.
 * @version 1.0.0
 * @license Apache 2 License
 */
public class MDOnly
{
   private static final String   MD_HEADER    = "MD  ";

   private String            _strucID;               /* Structure identifier */
   private int               _version;               /* Structure version number */

   private byte[]            _accountingToken;       /* Accounting token */
   private String            _applicationIdData;     /* Application data relating to identity */
   private String            _applicationOriginData; /* Application data relating to origin */
   private int               _backoutCount;          /* Backout counter */
   private int               _characterSet;          /* Character set identifier of message data */
   private byte[]            _correlationId;         /* Correlation identifier */
   private int               _encoding;              /* Numeric encoding of message data */
   private int               _expiry;                /* Message lifetime */
   private int               _feedback;              /* Feedback or reason code */
   private String            _format;                /* Format name of message data */
   private byte[]            _groupId;               /* Group identifier */
   private int               _messageFlags;          /* Message flags */
   private byte[]            _messageId;             /* Message identifier */
   private int               _messageSequenceNumber; /* Sequence number of logical message within group */
   private int               _messageType;           /* Message type */
   private int               _offset;                /* Offset of data in physical message from start of logical message */
   private int               _originalLength;        /* Length of original message */
   private int               _persistence;           /* Message persistence */
   private int               _priority;              /* Message priority */
   private String            _putApplicationName;    /* Name of application that put the message */
   private int               _putApplicationType;    /* Type of application that put the message */
   private GregorianCalendar _putDateTime;           /* Date when message was put */
   private String            _putDate;               /* Date when message was put */
   private String            _putTime;               /* Time when message was put */
   private String            _replyToQueueManagerName; /* Name of reply queue manager */
   private String            _replyToQueueName;      /* Name of reply queue */
   private int               _report;                /* Options for report messages */
   private String            _userId;                /* User identifier */
   //
   private int               _dataLength;            /* Message data length */

   /**
    * Instantiate the class and set default values for our internal fields.
    * @param mqMsg
    */
   public MDOnly()
   {
      super();
      //
      _strucID = MD_HEADER;
      _version = CMQC.MQMD_VERSION_2;
      _accountingToken = new byte[32];
      _applicationIdData = new String();
      _applicationOriginData = new String();
      _backoutCount = 0;
      _characterSet = CMQC.MQCCSI_DEFAULT;
      _correlationId = new byte[24];
      _encoding = CMQC.MQENC_NATIVE;
      _expiry = CMQC.MQEI_UNLIMITED;
      _feedback = CMQC.MQFB_NONE;
      _format = CMQC.MQFMT_STRING;
      _groupId = new byte[24];
      _messageFlags = CMQC.MQMF_NONE;
      _messageId = new byte[24];
      _messageSequenceNumber = 1;
      _messageType = CMQC.MQMT_DATAGRAM;
      _offset = 0;
      _originalLength = -1;
      _persistence = CMQC.MQPER_PERSISTENCE_AS_Q_DEF;
      _priority = CMQC.MQPRI_PRIORITY_AS_Q_DEF;
      _putApplicationName = new String();
      _putApplicationType = CMQC.MQAT_UNKNOWN;
      _putDateTime = new GregorianCalendar();
      _replyToQueueManagerName = new String();
      _replyToQueueName = new String();
      _report = CMQC.MQRO_NONE;
      _userId = new String();
      //
      _dataLength = 0;
   }

   /**
    * Instantiate the class and copy the incoming MQMD class fields to our internal fields.
    * @param mqMsg
    */
   public MDOnly(MQMessage mqMsg)
   {
      super();
      //
      _strucID = MD_HEADER;
      _version = mqMsg.getVersion();
      _accountingToken = mqMsg.accountingToken;
      _applicationIdData = mqMsg.applicationIdData;
      _applicationOriginData = mqMsg.applicationOriginData;
      _backoutCount = mqMsg.backoutCount;
      _characterSet = mqMsg.characterSet;
      _correlationId = mqMsg.correlationId;
      _encoding = mqMsg.encoding;
      _expiry = mqMsg.expiry;
      _feedback = mqMsg.feedback;
      _format = mqMsg.format;
      _groupId = mqMsg.groupId;
      _messageFlags = mqMsg.messageFlags;
      _messageId = mqMsg.messageId;
      _messageSequenceNumber = mqMsg.messageSequenceNumber;
      _messageType = mqMsg.messageType;
      _offset = mqMsg.offset;
      _originalLength = mqMsg.originalLength;
      _persistence = mqMsg.persistence;
      _priority = mqMsg.priority;
      _putApplicationName = mqMsg.putApplicationName;
      _putApplicationType = mqMsg.putApplicationType;
      _putDateTime = mqMsg.putDateTime;
      _replyToQueueManagerName = mqMsg.replyToQueueManagerName;
      _replyToQueueName = mqMsg.replyToQueueName;
      _report = mqMsg.report;
      _userId = mqMsg.userId;
      //
      _dataLength = mqMsg.getTotalMessageLength();
   }

   /**
    * Copy the incoming MQMD class fields to our internal fields.
    * @param mqMsg
    */
   public void copyFrom(MQMessage mqMsg)
   {
      _strucID = MD_HEADER;
      _version = mqMsg.getVersion();
      _accountingToken = mqMsg.accountingToken;
      _applicationIdData = mqMsg.applicationIdData;
      _applicationOriginData = mqMsg.applicationOriginData;
      _backoutCount = mqMsg.backoutCount;
      _characterSet = mqMsg.characterSet;
      _correlationId = mqMsg.correlationId;
      _encoding = mqMsg.encoding;
      _expiry = mqMsg.expiry;
      _feedback = mqMsg.feedback;
      _format = mqMsg.format;
      _groupId = mqMsg.groupId;
      _messageFlags = mqMsg.messageFlags;
      _messageId = mqMsg.messageId;
      _messageSequenceNumber = mqMsg.messageSequenceNumber;
      _messageType = mqMsg.messageType;
      _offset = mqMsg.offset;
      _originalLength = mqMsg.originalLength;
      _persistence = mqMsg.persistence;
      _priority = mqMsg.priority;
      _putApplicationName = mqMsg.putApplicationName;
      _putApplicationType = mqMsg.putApplicationType;
      _putDateTime = mqMsg.putDateTime;
      _replyToQueueManagerName = mqMsg.replyToQueueManagerName;
      _replyToQueueName = mqMsg.replyToQueueName;
      _report = mqMsg.report;
      _userId = mqMsg.userId;
      //
      _dataLength = mqMsg.getTotalMessageLength();
   }
   
   /**
    * Copy the current fields to the MQMessage parameter.
    * @param mqMsg
    */
   public void copyTo(MQMessage mqMsg)
   {
      try
      {
         mqMsg.setVersion(_version);
      }
      catch (MQException e)
      {
         try
         {
            mqMsg.setVersion(CMQC.MQMD_VERSION_2);
         }
         catch (MQException e1)
         {}
      }
      
      mqMsg.accountingToken        = _accountingToken;
      mqMsg.applicationIdData      = _applicationIdData;
      mqMsg.applicationOriginData  = _applicationOriginData;
      mqMsg.backoutCount           = _backoutCount;
      mqMsg.characterSet           = _characterSet;
      mqMsg.messageId              = _messageId;
      mqMsg.correlationId          = _correlationId;
      mqMsg.encoding               = _encoding;
      mqMsg.expiry                 = _expiry;
      mqMsg.feedback               = _feedback;
      mqMsg.format                 = _format;
      mqMsg.groupId                = _groupId;
      mqMsg.messageFlags           = _messageFlags;
      mqMsg.messageSequenceNumber  = _messageSequenceNumber;
      mqMsg.messageType            = _messageType;
      mqMsg.offset                 = _offset;
      mqMsg.originalLength         = _originalLength;
      mqMsg.persistence            = _persistence;
      mqMsg.priority               = _priority;
      mqMsg.putApplicationName     = _putApplicationName;
      mqMsg.putApplicationType     = _putApplicationType;
      mqMsg.putDateTime            = _putDateTime;
      mqMsg.replyToQueueManagerName= _replyToQueueManagerName;
      mqMsg.replyToQueueName       = _replyToQueueName;
      mqMsg.report                 = _report;
      mqMsg.userId                 = _userId;
   }
   
   /*************************
    * Getters and setters.
    *************************
    */

   /**
    * Retrieve the message's accounting token.
    * @return accountingToken
    */
   public byte[] getAccountingToken()
   {
      return _accountingToken;
   }

   /**
    * Retrieve the message's application identification data.
    * @return applicationIdData
    */
   public String getApplicationIdData()
   {
      return _applicationIdData;
   }

   /**
    * Retrieve the message's application origin data.
    * @return applicationOriginData
    */
   public String getApplicationOriginData()
   {
      return _applicationOriginData;
   }

   /**
    * Retrieve the message's backout count.
    * @return backoutCount
    */
   public int getBackoutCount()
   {
      return _backoutCount;
   }

   /**
    * Retrieve the message's CCSID. 
    * @return characterSet
    */
   public int getCharacterSet()
   {
      return _characterSet;
   }

   /**
    * Retrieve the message's correlation identification.
    * @return correlationId
    */
   public byte[] getCorrelationId()
   {
      return _correlationId;
   }

   /**
    * Retrieve the message's data length.
    * @return dataLength
    */
   public int getDataLength()
   {
      return _dataLength;
   }

   /**
    * Retrieve the message's encoding. 
    * @return encoding
    */
   public int getEncoding()
   {
      return _encoding;
   }

   /**
    * Retrieve the message's expiry.
    * @return expiry
    */
   public int getExpiry()
   {
      return _expiry;
   }

   /**
    * Retrieve the message's feedback code.
    * @return feedback
    */
   public int getFeedback()
   {
      return _feedback;
   }

   /**
    * Retrieve the message's format value.
    * @return format
    */
   public String getFormat()
   {
      return _format;
   }

   /**
    * Retrieve the message's group identification.
    * @return groupId
    */
   public byte[] getGroupId()
   {
      return _groupId;
   }

   /**
    * Retrieve the message's message flags.
    * @return messageFlags
    */
   public int getMessageFlags()
   {
      return _messageFlags;
   }

   /**
    * Retrieve the message's message identification.
    * @return messageId
    */
   public byte[] getMessageId()
   {
      return _messageId;
   }

   /**
    * Retrieve the message's message sequence number
    * @return messageSequenceNumber
    */
   public int getMessageSequenceNumber()
   {
      return _messageSequenceNumber;
   }

   /**
    * Retrieve the message's message type. 
    * @return messageType
    */ 
   public int getMessageType()
   {
      return _messageType;
   }

   /**
    * Retrieve the message's offset.
    * @return offset
    */
   public int getOffset()
   {
      return _offset;
   }

   /**
    * Retrieve the message's original length.
    * @return originalLength
    */
   public int getOriginalLength()
   {
      return _originalLength;
   }

   /**
    * Retrieve the message's persistence.
    * @return persistence
    */
   public int getPersistence()
   {
      return _persistence;
   }

   /**
    * Retrieve the message's priority.
    * @return priority
    */
   public int getPriority()
   {
      return _priority;
   }

   /**
    * Retrieve the message's put application name. 
    * @return putApplicationName
    */
   public String getPutApplicationName()
   {
      return _putApplicationName;
   }

   /**
    * Retrieve the message's put application type.
    * @return putApplicationType
    */
   public int getPutApplicationType()
   {
      return _putApplicationType;
   }

   /**
    * Retrieve the message's put date.
    * @return putDate
    */
   public String getPutDate()
   {
      return _putDate;
   }

   /**
    * Retrieve the message's put date & time value. 
    * @return _putDateTime
    */
   public GregorianCalendar getPutDateTime()
   {
      return _putDateTime;
   }

   /**
    * Retrieve the message's put time. 
    * @return putTime
    */
   public String getPutTime()
   {
      return _putTime;
   }

   /**
    * Retrieve the message's reply to queue manager name.
    * @return replyToQueueManagerName
    */
   public String getReplyToQueueManagerName()
   {
      return _replyToQueueManagerName;
   }

   /**
    * Retrieve the message's reply to queue name.
    * @return replyToQueueName
    */
   public String getReplyToQueueName()
   {
      return _replyToQueueName;
   }

   /**
    * Retrieve the message's report.
    * @return report
    */
   public int getReport()
   {
      return _report;
   }

   /**
    * Retrieve the message's structure identification.
    * @return strucID
    */
   public String getStrucID()
   {
      return _strucID;
   }

   /**
    * Retrieve the message's user identification.
    * @return userId
    */
   public String getUserId()
   {
      return _userId;
   }

   /**
    * Retrieve the message's version number. 
    * @return version
    */
   public int getVersion()
   {
      return _version;
   }

   /**
    * Set the MD's accounting token. 
    * @param token
    */
   public void setAccountingToken(byte[] token)
   {
      _accountingToken = token;
   }

   /**
    * Set the MD's application identification data.
    * @param applIdData
    */
   public void setApplicationIdData(String applIdData)
   {
      _applicationIdData = applIdData;
   }

   /**
    * Set the MD's application origin data. 
    * @param applOriginData
    */
   public void setApplicationOriginData(String applOriginData)
   {
      _applicationOriginData = applOriginData;
   }

   /**
    * Set the MD's backout count. 
    * @param count
    */
   public void setBackoutCount(int count)
   {
      _backoutCount = count;
   }

   /**
    * Set the MD's character set. 
    * @param ccsid
    */
   public void setCharacterSet(int ccsid)
   {
      _characterSet = ccsid;
   }

   /**
    * Set the MD's correlation identification.
    * @param correlID
    */
   public void setCorrelationId(byte[] correlID)
   {
      _correlationId = correlID;
   }

   /**
    * Set the MD's data length. 
    * @param len
    */
   public void setDataLength(int len)
   {
      _dataLength = len;
   }

   /**
    * Set the MD's encoding. 
    * @param encoding
    */
   public void setEncoding(int encoding)
   {
      _encoding = encoding;
   }

   /**
    * Set the MD's expiry. 
    * @param expiry
    */
   public void setExpiry(int expiry)
   {
      _expiry = expiry;
   }

   /**
    * Set the MD's feedback code. 
    * @param feedback
    */
   public void setFeedback(int feedback)
   {
      _feedback = feedback;
   }

   /**
    * Set the MD's format value. 
    * @param format
    */
   public void setFormat(String format)
   {
      _format = format;
   }

   /**
    * Set the MD's group identification.
    * @param groupId
    */
   public void setGroupId(byte[] groupId)
   {
      _groupId = groupId;
   }

   /**
    * Set the MD's message flags.
    * @param flags
    */
   public void setMessageFlags(int flags)
   {
      _messageFlags = flags;
   }

   /**
    * Set the MD'smessage identification.  
    * @param msgId
    */
   public void setMessageId(byte[] msgId)
   {
      _messageId = msgId;
   }

   /**
    * Set the MD's message sequence number. 
    * @param sequence
    */
   public void setMessageSequenceNumber(int sequence)
   {
      _messageSequenceNumber = sequence;
   }

   /**
    * Set the MD's message type. 
    * @param messageType
    */
   public void setMessageType(int messageType)
   {
      _messageType = messageType;
   }

   /**
    * Set the MD's offset.
    * @param offset
    */
   public void setOffset(int offset)
   {
      _offset = offset;
   }

   /**
    * Set the MD's original length. 
    * @param originalLength
    */
   public void setOriginalLength(int originalLength)
   {
      _originalLength = originalLength;
   }

   /**
    * Set the MD's persistence.
    * @param persistence
    */
   public void setPersistence(int persistence)
   {
      _persistence = persistence;
   }

   /**
    * Set the MD's priority.
    * @param priority
    */
   public void setPriority(int priority)
   {
      _priority = priority;
   }

   /**
    * Set the MD's put application name.
    * @param putApplName
    */
   public void setPutApplicationName(String putApplName)
   {
      _putApplicationName = putApplName;
   }

   /**
    * Set the MD's put application type.
    * @param putApplType
    */
   public void setPutApplicationType(int putApplType)
   {
      _putApplicationType = putApplType;
   }

   /**
    * Set the MD's put date.
    * @param putDate
    */
   public void setPutDate(String putDate)
   {
      _putDate = putDate;
   }

   /**
    * Set the MD's put date & time value.
    * @param calendar
    */
   public void setPutDateTime(GregorianCalendar calendar)
   {
      _putDateTime = calendar;
   }

   /**
    * Set the MD's put time.
    * @param putTime
    */
   public void setPutTime(String putTime)
   {
      _putTime = putTime;
   }

   /**
    * Set the MD's reply to queue manager name.
    * @param replyToQMgrName
    */
   public void setReplyToQueueManagerName(String replyToQMgrName)
   {
      _replyToQueueManagerName = replyToQMgrName;
   }

   /**
    * Set the MD's reply to queue name.
    * @param replyToQName
    */
   public void setReplyToQueueName(String replyToQName)
   {
      _replyToQueueName = replyToQName;
   }

   /**
    * Set the MD's report.  
    * @param report
    */
   public void setReport(int report)
   {
      _report = report;
   }

   /**
    * Set the MD's user identification.  
    * @param userId
    */
   public void setUserId(String userId)
   {
      _userId = userId;
   }

   /**
    * Set the MD's version number.  
    * @param version
    */
   public void setVersion(int version)
   {
      _version = version;
   }
}
